def ensure_scope(
    level: int, global_dict=..., local_dict=..., resolvers=..., target=..., **kwargs
) -> Scope: ...

class Scope:
    level = ...
    scope = ...
    target = ...
    resolvers = ...
    temps = ...
    def __init__(
        self, level, global_dict=..., local_dict=..., resolvers=..., target=...
    ) -> None: ...
    @property
    def has_resolvers(self) -> bool: ...
    def resolve(self, key: str, is_local: bool): ...
    def swapkey(self, old_key: str, new_key: str, new_value=...): ...
    def add_tmp(self, value) -> str: ...
    @property
    def ntemps(self) -> int: ...
    @property
    def full_scope(self): ...
