def set_use_numexpr(v: bool = ...) -> None: ...
def set_numexpr_threads(n=...) -> None: ...
def evaluate(op, a, b, use_numexpr: bool = ...): ...
def where(cond, a, b, use_numexpr: bool = ...): ...
def set_test_mode(v: bool = ...) -> None: ...
def get_test_result(): ...
