from typing import Any

ARITHMETIC_BINOPS: set[str] = ...
COMPARISON_BINOPS: set[str] = ...

def get_op_result_name(left: Any, right: Any): ...
def fill_binop(left: Any, right: Any, fill_value: Any): ...
