import numpy as np

from pandas.core.dtypes.generic import ABCExtensionArray as ABCExtensionArray

def arithmetic_op(left: np.ndarray | ABCExtensionArray, right, op, str_rep: str): ...
def comparison_op(
    left: np.ndarray | ABCExtensionArray, right, op
) -> np.ndarray | ABCExtensionArray: ...
def na_logical_op(x: np.ndarray, y, op): ...
def logical_op(
    left: np.ndarray | ABCExtensionArray, right, op
) -> np.ndarray | ABCExtensionArray: ...
def get_array_op(op, str_rep: str | None = ...): ...
